#ifndef COLORCOMBOBOX_H
#define COLORCOMBOBOX_H

#include <QComboBox>

class ColorComboBox : public QComboBox
{
    Q_OBJECT

public:
    ColorComboBox(QWidget* parent);
};

#endif // COLORCOMBOBOX_H
